#ifndef _FILTERING_H_
#define _FILTERING_H_

#include "spectrum.h"
#include "adjacency.h"

typedef struct _kernel {
  float *val;
  AdjRel *adj;
  int xsize,ysize;
} Kernel;

/* Funes dadas */

Kernel *MakeKernel(char *coefs); /* Cria e inicializa kernel a partir
                                    de cadeia de coeficientes */
Kernel *CreateKernel(AdjRel *A); /* Cria kernel sem coeficientes */
void    DestroyKernel(Kernel **K); /* Destri kernel */

Kernel *FoldKernel(Kernel *K); /* Reflete o kernel */

Image  *LinearFilter(Image *img, Kernel *K); /* Filtra a imagem por
						convoluo */


/*-------------------- Questes deste laboratrio ------------------------*/

/* Questo 1 (1,0): Calcule a multiplicao entre dois espectros de
   freqncia. */

Spectrum *FilterInFreq(Spectrum *spc1, Spectrum *spc2); 


/* Questo 2 (1,0): Gere o espectro real com ncols x nrows pixels de
   um filtro de Butterworth do tipo passa-baixas freqncias, onde Dl
   indica a freqncia de corte e n o fator de decaimento. Lembre-se
   que o espectro  peridico e pode ser gerado no centro da imagem e
   depois transladado para a origem. */

Spectrum *LowPassFilter(int ncols, int nrows, float Dl, float n);

/* Questo 3 (1,0): Gere o espectro real com ncols x nrows pixels de
   um filtro de Butterworth do tipo passa-altas freqncias, onde Dh
   indica a freqncia de corte e n o fator de decaimento. */

Spectrum *HighPassFilter(int ncols, int nrows, float Dh, float n);

/* Questo 4 (1,0): Gere o espectro real com ncols x nrows pixels de
   um filtro de Butterworth do tipo passa-faixa, onde Dl e Dh indicam
   as freqncias de corte do passa-baixas e do passa-altas,
   respectivamente, e nl e nh os seus fatores de decaimento. */

Spectrum *BandPassFilter(int ncols, int nrows, float Dl, float nl, float Dh, 
float nh);

/* Questo 5 (1,0): Gere o espectro real com ncols x nrows pixels de
   um filtro de Butterworth do tipo rejeita-faixa, onde Dl e Dh
   indicam as freqncias de corte do passa-baixas e do passa-altas,
   respectivamente, e nl e nh os seus fatores de decaimento. */

Spectrum *BandRejectFilter(int ncols, int nrows, float Dl, float nl, float 
Dh, float nh);


/* Questo 6 (3,0): Aplique a inversa de Moore-Penrose para transformar
   um espectro com (N+1)x(N+1) pixels em uma mscara com (n+1)x(n+1)
   coeficientes. */

Spectrum *MoorePenrose(int n, int N); /* Questo 6a (1,5): Gere a
					 matriz inversa generalizada
					 de Moore-Penrose. */ 


Kernel *ComputeMask(Spectrum *spc, Spectrum *moore, int n); /* Questo
                                             6b (1,5): Gere o kernel
                                             com (n+1)x(n+1)
                                             coeficientes a partir do
                                             espectro spc de um filtro
                                             com (N+1)x(N+1)
                                             pixels. */


/* Questo 7 (2,0): Escolha um dos filtros R(u,v) vistos na aula sobre
   restaurao de imagens (e.g. correo de foco ou correo de
   movimento relativo entre cmera e objeto) e gere o seu espectro de
   freqncia para demonstrar o seu resultado quando aplicado a uma
   imagem degradada. OS DOIS PONTOS DESTA QUESTO INCLUEM A
   DEMONSTRAO CORRETA DOS RESULTADOS. */

Spectrum *RestaurationFilter(int ncols, int nrows);


#endif
