#include "gift.h"

Image *Gradient(Image *img)
{
  Image *grad;
  Pixel s,u,v,g;
  int s1,u1,v1;

  grad = CreateImage(img->ncols,img->nrows);
  for (u.y=0; u.y < grad->nrows; u.y++) 
    for (u.x=0; u.x < grad->ncols; u.x++) {
      v.x = ((u.x + 1)==grad->ncols)?u.x:u.x+1; 
      v.y = u.y;
      s.x = ((u.x-1)==-1)?u.x:u.x-1;
      s.y = u.y;
      u1  = u.x + grad->tbrow[u.y];
      v1  = v.x + grad->tbrow[v.y];
      s1  = s.x + grad->tbrow[s.y];
      g.x = ((img->val[u1]+img->val[v1])/2 - 
	     (img->val[u1]+img->val[s1])/2);  
      v.x = u.x;
      v.y = ((u.y + 1)==grad->nrows)?u.y:u.y+1; 
      s.x = u.x;
      s.y = ((u.y-1)==-1)?u.y:u.y-1;
      u1  = u.x + grad->tbrow[u.y];
      v1  = v.x + grad->tbrow[v.y];
      s1  = s.x + grad->tbrow[s.y];
      g.y = ((img->val[u1]+img->val[v1])/2 - 
	     (img->val[u1]+img->val[s1])/2);  
      grad->val[u1] = (int)sqrt(g.x*g.x + g.y*g.y);
    }
  return(grad);
}

int main(int argc, char **argv)
{
  timer tic,toc;
  FILE *ftime=fopen("time.txt","w");
  AdjRel *A=NULL;
  Image *img,*lambda,*handicap;
  Image *aux1,*aux2,*grad;


  /* The following block must the remarked when using non-linux machines 

  void *trash = malloc(1);                 
  struct mallinfo info;   
  int MemDinInicial, MemDinFinal;
  free(trash); 
  info = mallinfo();
  MemDinInicial = info.uordblks;
  */

  /* Watershed de marcadores rotulados. O resultado deve ser igual ao
     do programa watershed.c */

  img      = ReadImage("../data/caudate.pgm");
  aux1     = GaussStretch(img,140.0,30.0);
  A        = Circular(1.5);
  grad     = Gradient(aux1);
  lambda   = LambdaInit(img->ncols,img->nrows);  
  lambda->val[45+img->tbrow[37]] =1;
  lambda->val[82+img->tbrow[44]] =2;
  gettimeofday(&tic,NULL);  
  WaterLabel(grad,lambda);
  gettimeofday(&toc,NULL);
  fprintf(ftime,"WaterLabel in %f milliseconds\n",(toc.tv_sec-tic.tv_sec)*1000.0 + (toc.tv_usec-tic.tv_usec)*0.001);
  aux2     = DrawBorder(img,lambda,255);
  WriteImage(aux2,"questao3_a.pgm");
  DestroyImage(&aux1);
  DestroyImage(&aux2);
  DestroyImage(&lambda);
  DestroyImage(&img);
  DestroyAdjRel(&A);

  /* Watershed com reconstrucao local */

  img      = ReadImage("../data/carcinoma.pgm");
  A        = Circular(1.5);
  lambda   = LambdaInit(img->ncols,img->nrows);  
  handicap = CreateImage(img->ncols,img->nrows);  
  SetImage(handicap,INT_MAX);
  handicap->val[119+img->tbrow[116]]=30;
  lambda->val[119+img->tbrow[116]]   =0;
  gettimeofday(&tic,NULL);  
  LocalWaterRec(img,handicap,lambda);
  gettimeofday(&toc,NULL);
  fprintf(ftime,"LocalWaterRec in %f milliseconds\n",(toc.tv_sec-tic.tv_sec)*1000.0 + (toc.tv_usec-tic.tv_usec)*0.001);
  aux2     = DrawBorder(img,lambda,255);
  WriteImage(aux2,"questao3_b.pgm");
  DestroyImage(&lambda);
  DestroyImage(&handicap);
  DestroyImage(&aux2);
  DestroyImage(&img);
  DestroyAdjRel(&A);

  /* Watershed de marcadores com limiar. O resultado deve ser parecido
     com o anterior. */

  img      = ReadImage("../data/carcinoma.pgm");
  A        = Circular(1.5);
  lambda   = LambdaInit(img->ncols,img->nrows);  
  lambda->val[119+img->tbrow[116]] = 0;
  gettimeofday(&tic,NULL);  
  WaterThres(img,lambda,30);
  gettimeofday(&toc,NULL);
  fprintf(ftime,"WaterThres in %f milliseconds\n",(toc.tv_sec-tic.tv_sec)*1000.0 + (toc.tv_usec-tic.tv_usec)*0.001);
  aux2     = DrawBorder(img,lambda,255);
  WriteImage(aux2,"questao3_c.pgm");
  DestroyImage(&lambda);
  DestroyImage(&aux2);
  DestroyImage(&img);
  DestroyAdjRel(&A);

  /* The following block must the remarked when using non-linux machines 

  info = mallinfo();
  MemDinFinal = info.uordblks;
  if (MemDinInicial!=MemDinFinal)
    printf("\n\nDinamic memory was not completely deallocated (%d, %d)\n",
	   MemDinInicial,MemDinFinal);   
  */

  fclose(ftime);

  return(0);
}
